%feature("docstring") OT::Analytical
"Base class to find the design point.

Available constructors:
    Analytical(*nearestPointAlgorithm, event, physicalStartingPoint*)

Parameters
----------
nearestPointAlgorithm : :class:`~openturns.NearestPointAlgorithm`
    Optimization algorithm used to research the design point.
event : :class:`~openturns.Event`
    Failure event.
physicalStartingPoint : float sequence
    Starting point of the optimization algorithm, declared in the physical
    space.

Notes
-----
An event is defined as follows:
:math:`\\\\cD_f = \\\\{\\\\vect{X} \\\\in \\\\Rset^n \\\\, / \\\\, g(\\\\vect{X},\\\\vect{d}) \\\\le 0\\\\}`
where :math:`\\\\vect{X}` denotes a random input vector, representing the sources
of uncertainties, :math:`\\\\vect{d}` is a determinist vector, representing the
fixed variables and :math:`g(\\\\vect{X},\\\\vect{d})` is the limit state function of
the model.
The probability content of the event :math:`\\\\cD_f`:

.. math::

    P_f = \\\\int_{g(\\\\vect{X},\\\\vect{d})\\\\le 0}f_\\\\vect{X}(\\\\vect{x})d\\\\vect{x}

may be evaluated with the :class:`~openturns.FORM` or :class:`~openturns.SORM`
methods.

In order to evaluate an approximation of :math:`P_f`, these analytical methods
uses the Nataf isoprobabilistic transformation which maps the probabilistic
model in terms of :math:`\\\\vect{X}` onto an equivalent model in terms of
:math:`n` independant standard normal random :math:`\\\\vect{U}`. In that new
:math:`\\\\vect{u}`-space, the event has the new expression defined from the
transformed limit state function of the model
:math:`G : \\\\cD_f = \\\\{\\\\vect{U} \\\\in \\\\Rset^n \\\\, / \\\\, G(\\\\vect{U}\\\\,,\\\\,\\\\vect{d}) \\\\le 0\\\\}`
and its boundary: :math:`\\\\{\\\\vect{U} \\\\in \\\\Rset^n \\\\, / \\\\,G(\\\\vect{U}\\\\,,\\\\,\\\\vect{d}) = 0\\\\}`.

These analytical methods rely on the assumption that most of the contribution
to :math:`P_f` comes from points located in the vicinity of a particular point
:math:`P^*`, the **design point**, defined in the :math:`\\\\vect{u}`-space as the
point located on the limit state surface and of maximal likelihood.
Given the probabilistic caracteristics of the :math:`\\\\vect{u}`-space,
:math:`P^*` has a geometrical interpretation: it is the point located on the
event boundary and at minimal distance from the center of the
:math:`\\\\vect{u}`-space. Thus, the design point :math:`P^*` is the result of a
constrained optimization problem.

See also
--------
FORM, SORM, StrongMaximumTest

Examples
--------
>>> import openturns as ot
>>> myFunction = ot.NumericalMathFunction(['E', 'F', 'L', 'I'], ['d'], ['-F*L^3/(3*E*I)'])
>>> myDistribution = ot.Normal([50., 1., 10., 5.], [1.]*4, ot.IdentityMatrix(4))
>>> vect = ot.RandomVector(myDistribution)
>>> output = ot.RandomVector(myFunction, vect)
>>> myEvent = ot.Event(output, ot.Less(), -3.0)
>>> # We create a NearestPoint algorithm
>>> myCobyla = ot.Cobyla()
>>> myAlgo = ot.Analytical(myCobyla, myEvent, [50., 1., 10., 5.])"

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::getAnalyticalResult
"Accessor to the result.

Returns
-------
result : :class:`~openturns.AnalyticalResult`
    Result structure which contains the results of the optimisation problem."

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::getNearestPointAlgorithm
"Accessor to the optimization algorithm used to find the design point.

Returns
-------
algorithm : :class:`~openturns.NearestPointAlgorithm`
     Optimization algorithm used to research the design point."

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::setNearestPointAlgorithm
"Accessor to the optimization algorithm used to find the design point.

Parameters
----------
algorithm : :class:`~openturns.NearestPointAlgorithm`
     Optimization algorithm used to research the design point."

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::getPhysicalStartingPoint
"Accessor to the starting point of the optimization algorithm.

Returns
-------
point : float sequence
    Starting point of the optimization algorithm, declared in the physical
    space."

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::setPhysicalStartingPoint
"Accessor to the starting point of the optimization algorithm.

Parameters
----------
point : float sequence
    Starting point of the optimization algorithm, declared in the physical
    space."

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::getEvent
"Accessor to the event of which the probability is calculated.

Returns
-------
event : :class:`~openturns.Event`
    Event of which the probability is calculated."

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::setEvent
"Accessor to the event of which the probability is calculated.

Parameters
----------
event : :class:`~openturns.Event`
    Event of which the probability is calculated."

// ---------------------------------------------------------------------

%feature("docstring") OT::Analytical::run
"Perform the research of the design point.

Notes
-----
Performs the research of the design point and creates a
:class:`~openturns.AnalyticalResult`, the structure result which is
accessible with the method getAnalyticalResult.

See also
--------
getAnalyticalResult"
