%define OT_DynamicalFunction_doc
"Dynamical function.

Available constructors:
    DynamicalFunction(*meshDimension=1*)

    DynamicalFunction(*function, meshDimension=1*)

Parameters
----------
function : :class:`~openturns.NumericalMathFunction`
    Function :math:`g: \\\\Rset^d \\\\mapsto \\\\Rset^q` used to define a
    :class:`~openturns.SpatialFunction` object.
meshDimension : int, :math:`n \\\\geq 0`
    Dimension of the mesh :math:`\\\\cM`.

Notes
-----
Dynamical functions are functions being able to act on fields. OpenTURNS
defines two particular dynamical functions:
the :class:`spatial function <openturns.SpatialFunction>` and
the :class:`temporal function <openturns.TemporalFunction>`.


A dynamical function
:math:`f_{dyn}:\\\\cD \\\\times \\\\Rset^d \\\\mapsto \\\\cD' \\\\times \\\\Rset^q`
where :math:`\\\\cD \\\\in \\\\Rset^n` and :math:`\\\\cD' \\\\in \\\\Rset^p` is defined by:

.. math::

    f_{dyn}(\\\\vect{t}, \\\\vect{x}) = (t'(\\\\vect{t}), v'(\\\\vect{t}, \\\\vect{x}))

with :math:`t': \\\\cD \\\\mapsto \\\\cD'` and
:math:`v': \\\\cD \\\\times \\\\Rset^d \\\\mapsto \\\\Rset^q`.

A dynamical function :math:`f_{dyn}` transforms a multivariate stochastic
process:

.. math::

    X: \\\\Omega \\\\times \\\\cD \\\\mapsto \\\\Rset^d

where :math:`\\\\cD \\\\in \\\\Rset^n` is discretized according to the mesh :math:`\\\\cM`
into the multivariate stochastic process:

.. math::

    Y=f_{dyn}(X)

such that:

.. math::

    Y: \\\\Omega \\\\times \\\\cD' \\\\mapsto \\\\Rset^q

where the mesh :math:`\\\\cD' \\\\in \\\\Rset^p`  is discretized according to the
:math:`\\\\cM'`.

A dynamical function :math:`f_{dyn}` also acts on fields and produces fields of
possibly different dimension (:math:`q\\\\neq d`) and mesh (:math:`\\\\cD \\\\neq \\\\cD'`
or :math:`\\\\cM \\\\neq \\\\cM'`).

OpenTURNS only proposes dynamical functions where :math:`\\\\cD'=\\\\cD` and
:math:`\\\\cM'=\\\\cM` which means that :math:`t'=Id` through the *spatial function*
and the *temporal function*. It follows that the process :math:`Y` shares the
same mesh with :math:`X`, only its values have changed."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation
OT_DynamicalFunction_doc

// ---------------------------------------------------------------------

%define OT_DynamicalFunction_getCallsNumber_doc
"Get the number of calls of a DynamicalFunction.

Returns
-------
callsNumber : int
    Counts the number of times the DynamicalFunction has been called since its
    creation."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation::getCallsNumber
OT_DynamicalFunction_getCallsNumber_doc

// ---------------------------------------------------------------------

%define OT_DynamicalFunction_getInputDescription_doc
"Get the description of the inputs.

Returns
-------
inputDescription : list of strings
    Describes the inputs of the dynamical function."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation::getInputDescription
OT_DynamicalFunction_getInputDescription_doc

// ---------------------------------------------------------------------

%define OT_DynamicalFunction_getInputDimension_doc
"Get the dimension of the input.

Returns
-------
d : int
    Input dimension :math:`d` of the dynamical function."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation::getInputDimension
OT_DynamicalFunction_getInputDimension_doc

// ---------------------------------------------------------------------

%define OT_DynamicalFunction_getMarginal_doc
"Get the marginal(s) at given indice(s).

Parameters
----------
i : int or list of ints, :math:`0 \\\\leq i < d`
    Indice(s) of the marginal(s) needed. :math:`d` is the dimension of the
    DynamicalFunction.

Returns
-------
dynamicalFunction : :class:`~openturns.DynamicalFunction`
    DynamicalFunction restricted to the concerned marginal(s) at the indice(s)
    :math:`i` of the dynamical function :math:`f_{dyn}`."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation::getMarginal
OT_DynamicalFunction_getMarginal_doc

// ---------------------------------------------------------------------

%define OT_DynamicalFunction_getSpatialDimension_doc
"Get the dimension of the mesh.

Returns
-------
spatialDimension : int, :math:`n \\\\geq 0`
    Dimension of the mesh :math:`\\\\cM`."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation::getSpatialDimension
OT_DynamicalFunction_getSpatialDimension_doc

// ---------------------------------------------------------------------

%define OT_DynamicalFunction_getOutputDescription_doc
"Get the description of the outputs.

Returns
-------
outputDescription : list of strings
    Describes the outputs of the dynamical function."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation::getOutputDescription
OT_DynamicalFunction_getOutputDescription_doc

// ---------------------------------------------------------------------

%define OT_DynamicalFunction_getOutputDimension_doc
"Get the dimension of the output.

Returns
-------
q : int
    Output dimension :math:`q` of the dynamical function."
%enddef
%feature("docstring") OT::DynamicalFunctionImplementation::getOutputDimension
OT_DynamicalFunction_getOutputDimension_doc

// ---------------------------------------------------------------------

%feature("docstring") OT::DynamicalFunctionImplementation::setInputDescription
"Set the description of the inputs.

Parameters
----------
inputDescription : list of strings
    Describes the inputs of the dynamical function."

// ---------------------------------------------------------------------

%feature("docstring") OT::DynamicalFunctionImplementation::setOutputDescription
"Set the description of the outputs.

Parameters
----------
outputDescription : list of strings
    Describes the outputs of the dynamical function."