%feature("docstring") OT::KrigingAlgorithm
"Kriging algorithm.

Available constructors:
    KrigingAlgorithm(*inputSample, outputSample, basis, covarianceModel, normalize=True*)

    KrigingAlgorithm(*inputSample, inputDistribution, outputSample, basis, covarianceModel*)

Parameters
----------
inputSample, outputSample : 2D float sequences
    The input and output samples of a model evaluated apart.
inputDistribution : :class:`~openturns.Distribution`
    Joint probability density function of the physical input vector.
    If *inputDistribution* is not specified, the *inputSample* is assumed
    to be normally-distributed.
basis : :class:`~openturns.Basis`
    Basis of the output of the regression.
covarianceModel : :class:`~openturns.CovarianceModel`
    Correlation function.
normalize : bool, optional
    If *True*, *inputSample* is normalized.

Notes
-----
Kriging (also known as Gaussian process regression) is a Bayesian technique
that aim at approximating functions (most often in order to surrogate it
because it is expensive to evaluate). In the following it is assumed we aim at
surrogating a scalar-valued model :math:`\\\\cM: \\\\vect{x} \\\\mapsto y`. Note the
OpenTURNS implementation of Kriging can deal with vector-valued functions
(:math:`\\\\cM: \\\\vect{x} \\\\mapsto \\\\vect{y}`), but it simply loops over each output.
It is also assumed the model was runned over a design of experiments in order
to produce a set of observations gathered in the following dataset:
:math:`\\\\left(\\\\left(\\\\vect{x}^{(i)}, y^{(i)}\\\\right), i = 1, \\\\ldots, m\\\\right)`.
Ultimately Kriging aims at producing a predictor (also known as a response
surface or metamodel) denoted as :math:`\\\\tilde{\\\\cM}`.

We put the following Gaussian process prior on the model :math:`\\\\cM`:

.. math::

    Y(\\\\vect{x}) = \\\\Tr{\\\\vect{f}(\\\\vect{x})} \\\\vect{\\\\beta} + Z(\\\\vect{x})

where :math:`\\\\Tr{\\\\vect{f}(\\\\vect{x})} \\\\vect{\\\\beta}` is a generalized linear
model based upon a functional basis
:math:`\\\\vect{f} = \\\\left(f_j, j = 1, \\\\ldots, p\\\\right)` and a vector of
coefficients :math:`\\\\vect{\\\\beta} = \\\\left(\\\\beta_j, j = 1, \\\\ldots, p\\\\right)`.
:math:`Z(\\\\vect{x})` is a zero-mean gaussian process with a stationary
autocorrelation function depending on the distance between samples:

.. math::

    \\\\mathbb{E}[Z(\\\\vect{x}), Z(\\\\vect{x'})] = \\\\sigma^2 R(\\\\vect{x} - \\\\vect{x'}, \\\\vect{\\\\theta})

where :math:`\\\\sigma^2 > 0` is the variance and :math:`R` is the correlation
function that solely depends on the Manhattan distance between input points
:math:`\\\\vect{x} - \\\\vect{x'}` and a vector of parameters
:math:`\\\\vect{\\\\theta} \\\\in \\\\Rset^{n_\\\\theta}`.

Examples
--------
>>> import openturns as ot
>>> f = ot.NumericalMathFunction(['x0'], ['f0'], ['x0 * sin(x0)'])
>>> inputSample = ot.NumericalSample([[1.], [3.], [5.], [6.,], [7.], [8.]])
>>> outputSample = f(inputSample)
>>> # Create the algorithm
>>> basis = ot.ConstantBasisFactory().build()
>>> covarianceModel = ot.SquaredExponential(1)
>>> algo = ot.KrigingAlgorithm(inputSample, outputSample, basis, covarianceModel)
>>> algo.run()
>>> # Get the results
>>> result = algo.getResult()
>>> metamodel = result.getMetaModel()"

// ---------------------------------------------------------------------

%feature("docstring") OT::KrigingAlgorithm::getOptimizer
"Get the solver used to optimize the covariance model parameters.

Returns
-------
algorithm : :class:`~openturns.BoundConstrainedAlgorithm`
    Solver used to optimize the covariance model parameters."

// ---------------------------------------------------------------------

%feature("docstring") OT::KrigingAlgorithm::setOptimizer
"Set the solver used to optimize the covariance model parameters.

Parameters
----------
algorithm : :class:`~openturns.BoundConstrainedAlgorithm`
    Solver used to optimize the covariance model parameters."

// ---------------------------------------------------------------------

%feature("docstring") OT::KrigingAlgorithm::getConditionalCovarianceModel
"Get the covariance model obtained after likelihood optimization.

Returns
-------
covarianceModel : :class:`~openturns.CovarianceModel`
    Covariance model obtained after likelihood optimization when a solver is
    provided."

// ---------------------------------------------------------------------

%feature("docstring") OT::KrigingAlgorithm::getInputTransformation
"Get the function normalizing the input.

Returns
-------
transformation : :class:`~openturns.NumericalMathFunction`
    Function normalizing the input."

// ---------------------------------------------------------------------

%feature("docstring") OT::KrigingAlgorithm::setInputTransformation
"Set the function normalizing the input.

Parameters
----------
transformation : :class:`~openturns.NumericalMathFunction`
    Function normalizing the input."

// ---------------------------------------------------------------------

%feature("docstring") OT::KrigingAlgorithm::getResult
"Get the results of the metamodel computation.

Returns
-------
result : :class:`~openturns.MetaModelResult`
    Structure containing all the results obtained after metamodel computation
    and created by the method :py:meth:`run`."
